@@ -19,8 +19,8 @@
 #
 
 from dbus.mainloop.glib import DBusGMainLoop
+from gi.repository import GLib, GObject, Gdk, Gtk
 import json
-import gobject, gtk
 from gwibber.microblog.util.const import *
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
@@ -39,30 +39,30 @@ if hasattr(gettext, 'bind_textdomain_codeset'):
     gettext.bind_textdomain_codeset('gwibber','UTF-8')
 gettext.textdomain('gwibber')
 
-class GwibberPosterVBox(gtk.VBox):
+class GwibberPosterVBox(Gtk.VBox):
   def __init__(self, content=None):
-    gtk.VBox.__init__(self)
+    Gtk.VBox.__init__(self)
     DBusGMainLoop(set_as_default=True)
-    loop = gobject.MainLoop()
+    loop = GLib.MainLoop()
     self.service = gwibber.microblog.util.getbus("Service")
 
     self.input = Input(content=content)
     self.input.connect("submit", self.on_input_activate)
     self.input.connect("changed", self.on_input_changed)
-    self.input_splitter = gtk.VPaned()
+    self.input_splitter = Gtk.VPaned()
     self.input_splitter.add1(self.input)
 
-    self.button_send = gtk.Button(_("Send"))
+    self.button_send = Gtk.Button(_("Send"))
     self.button_send.connect("clicked", self.on_button_send_clicked)
     self.message_target = AccountTargetBar()
     self.message_target.pack_end(self.button_send, False)
 
-    content = gtk.VBox(spacing=5)
+    content = Gtk.VBox(spacing=5)
     content.pack_start(self.input_splitter, True)
     content.pack_start(self.message_target, False)
     content.set_border_width(5)
 
-    layout = gtk.VBox()
+    layout = Gtk.VBox()
     layout.pack_start(content, True)
     self.add(layout)
 
@@ -77,13 +77,13 @@ class GwibberPosterVBox(gtk.VBox):
     self.service.SendMessage(self.input.get_text())
     self.input.clear()
 
-class AccountTargetBar(gtk.HBox):
+class AccountTargetBar(Gtk.HBox):
   __gsignals__ = {
-      "canceled": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
+      "canceled": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, ())
   }
 
   def __init__(self):
-    gtk.HBox.__init__(self, spacing=5)
+    Gtk.HBox.__init__(self, spacing=5)
     
     self.accounts = gwibber.microblog.util.getbus("Accounts")
     self.daemon = gwibber.microblog.util.getbus("Service")
@@ -92,7 +92,7 @@ class AccountTargetBar(gtk.HBox):
     self.accounts.connect_to_signal("Updated", self.on_account_changed)
     self.account_list = []
 
-    self.targetbar = gtk.HBox(False, 6)
+    self.targetbar = Gtk.HBox(False, 6)
     self.targetbar.set_size_request(0, 24)
     self.pack_start(self.targetbar, True)
 
@@ -118,10 +118,10 @@ class AccountTargetBar(gtk.HBox):
         self.account_list.append(account)
 
     for account in self.account_list:
-      img = gtk.Image ()
+      img = Gtk.Image ()
       img.set_from_file(resources.get_ui_asset("icons/breakdance/16x16/" + account["service"] + ".png"))
       img.show();
-      but = gtk.ToggleButton ()
+      but = Gtk.ToggleButton ()
       but.set_active (account["send_enabled"])
       if not account["send_enabled"]:
         but.set_tooltip_text (account["service"] + "(" + account["username"] + ") - " + _("Disabled"))
@@ -139,23 +139,23 @@ class AccountTargetBar(gtk.HBox):
     if account["send_enabled"] != w.get_active ():
       self.accounts.SendEnabled(id)
 
-class Input(gtk.Frame):
+class Input(Gtk.Frame):
   __gsignals__ = {
-    "submit": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, int)),
-    "changed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, int)),
-    "clear": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
+    "submit": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, (str, int)),
+    "changed": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, (str, int)),
+    "clear": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, ())
   }
 
   def __init__(self, content=None):
-    gtk.Frame.__init__(self)
+    Gtk.Frame.__init__(self)
 
     self.textview = InputTextView(content=content)
     self.textview.connect("submit", self.do_submit_event)
     self.textview.connect("clear", self.do_clear_event)
     self.textview.get_buffer().connect("changed", self.do_changed_event)
 
-    scroll = gtk.ScrolledWindow()
-    scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+    scroll = Gtk.ScrolledWindow()
+    scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
     scroll.add(self.textview)
     self.add(scroll)
 
@@ -185,14 +185,14 @@ class Input(gtk.Frame):
     self.emit("submit", text, chars)
 
 
-class InputTextView(gtk.TextView):
+class InputTextView(Gtk.TextView):
   __gsignals__ = {
-    "submit": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ()),
-    "clear": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
+    "submit": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, ()),
+    "clear": (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, ())
   }
 
   def __init__(self, content=None):
-    gtk.TextView.__init__(self)
+    Gtk.TextView.__init__(self)
     self.drawable = None
 
     self.content = content
@@ -216,14 +216,14 @@ class InputTextView(gtk.TextView):
     self.set_accepts_tab(True)
     self.set_editable(False)
     self.set_cursor_visible(True)
-    self.set_wrap_mode(gtk.WRAP_WORD_CHAR)
+    self.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
     self.set_left_margin(2)
     self.set_right_margin(2)
     self.set_pixels_above_lines(2)
     self.set_pixels_below_lines(2)
 
-    self.base_color = util.get_style().base[gtk.STATE_NORMAL]
-    self.error_color = gtk.gdk.color_parse("indianred")
+    self.base_color = util.get_style().base[Gtk.StateType.NORMAL]
+    self.error_color = Gdk.Color.parse("indianred")
 
     # set state online/offline
     if not self.connection.isConnected():
@@ -282,7 +282,7 @@ class InputTextView(gtk.TextView):
 
   def expose_view(self, window, event):
     if not self.drawable:
-      self.drawable = self.get_window(gtk.TEXT_WINDOW_TEXT)
+      self.drawable = self.get_window(Gtk.TextWindowType.TEXT)
       self.pango_overlay = self.create_pango_layout("")
       self.set_overlay_text(MAX_MESSAGE_LENGTH)
 
@@ -298,7 +298,7 @@ class InputTextView(gtk.TextView):
   def on_text_changed(self, w):
     chars = self.get_char_count()
     color = self.error_color if chars > MAX_MESSAGE_LENGTH else self.base_color
-    self.modify_base(gtk.STATE_NORMAL, color)
+    self.modify_base(Gtk.StateType.NORMAL, color)
 
   def on_connection_online(self, *args):
     self.set_sensitive(True)
