@@ -3,12 +3,12 @@
 
 import traceback, json, random, string
 import dbus, dbus.service
-import sqlite3, mx.DateTime, re, uuid
+import sqlite3, re, uuid
 import urlshorter, storage, network, util, uploader
 from gettext import lgettext as _
 
 from gi.repository import GLib
-from threading import Thread
+from threading import Thread, Lock
 from datetime import datetime, timedelta
 
 import logging
@@ -24,6 +24,8 @@ except:
   Dbusmenu = None
 
 from gi.repository import Gio
+from gi.repository import Accounts
+import collections
 
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
@@ -33,9 +35,9 @@ except:
   pass
 
 try:
-  from gi.repository import Indicate as indicate
+  from gi.repository import MessagingMenu
 except:
-  indicate = None
+  MessagingMenu = None
 
 GLib.threads_init()
 
@@ -136,27 +138,81 @@ class perform_operation(Thread):
     logger.debug("%s Finished operation (%s)" % (logtext, str(self.t_runtime)))
 
 
+class Account(collections.MutableMapping):
+  def __init__(self, account_service):
+    self._account_service = account_service
+    self._dict = {}
+    auth_data = account_service.get_auth_data()
+    auth = {
+      "id": auth_data.get_credentials_id(),
+      "method": auth_data.get_method(),
+      "mechanism": auth_data.get_mechanism(),
+      "parameters": auth_data.get_parameters(),
+    }
+    self._dict["auth"] = auth
+
+    # id
+    account = self._account_service.get_account()
+    self._account_id = account.id
+    service_name = self._account_service.get_service().get_name()
+    self._dict["id"] = "%s/%s" % (self._account_id, service_name)
+
+    # service -- pray that the provider name in libaccounts matches the name of
+    # our plugin
+    self._dict["service"] = account.get_provider_name()
+
+    self._account_service.connect("changed", self.on_account_changed, account)
+    self.on_account_changed (account_service, account)
+
+    # Create a lock for the login operation
+    self.login_lock = Lock()
+
+  def on_account_changed(self, account_service, account):
+    iterator = account.get_settings_iter(None)
+    settings = []
+    (ok, key, value) = iterator.next()
+    while ok:
+      settings.append((key, value))
+      (ok, key, value) = iterator.next()
+
+    settings.sort()
+    for (key, value) in settings:
+      if key.startswith("gwibber"):
+        self._dict[key.split("/")[1]] = value
+
+
+  def global_id(self):
+    return self._account_id
+
+  def enabled(self):
+    return self._account_service.get_enabled()
+
+  def __eq__(self, other):
+    return self._account_service == other._account_service
+
+  def __len__(self):
+    return len(self._dict)
+  def __getitem__(self, key):
+    return self._dict[key]
+  def __setitem__(self, key, value):
+    self._dict[key] = value
+  def __delitem__(self, key):
+    del self._dict[key]
+  def __contains__(self, key):
+    return key in self._dict
+  def __iter__(self):
+    return self._dict.iteritems()
+
+
 class OperationCollector:
   def __init__(self, dispatcher):
     self.dispatcher = dispatcher
 
-  def get_passwords(self, acct):
-    for key, val in acct.items():
-      if hasattr(val, "startswith") and val.startswith(":KEYRING:"):
-        id = "%s/%s" % (acct["id"], key)
-        try:
-          acct[key] = self.dispatcher.accounts.passwords[id]
-        except:
-          pass
-    return acct 
-
   def get_accounts(self):
-    data = json.loads(self.dispatcher.accounts.List())
-    return [self.get_passwords(acct) for acct in data]
+    return self.dispatcher.get_accounts()
 
   def get_account(self, id):
-    data = json.loads(self.dispatcher.accounts.Get(id))
-    return self.get_passwords(data)
+    return self.dispatcher.get_account(id)
 
   def handle_max_id(self, acct, opname, id=None):
     if not id: id = acct["id"]
@@ -179,17 +235,19 @@ class OperationCollector:
     return {}
 
   def validate_operation(self, acct, opname, enabled="receive_enabled"):
-    # if account doesn't have the required feature or is disabled, return
-    if enabled in acct:
-      if not acct[enabled]: return False
-    else: 
-      return False
+
+    if not acct.enabled(): return False
+
     # if there is an account for a service that gwibber doesn't no about, return
     if not acct["service"] in SERVICES: return False
     service = SERVICES[acct["service"]]
+
+    if enabled in acct:
+      if not acct[enabled]: return False
+
     return acct["service"] in PROTOCOLS and \
            opname in service["features"] and \
-           opname in FEATURES and acct[enabled]
+           opname in FEATURES
 
   def stream_to_operation(self, stream):
     try:
@@ -260,7 +318,6 @@ class Dispatcher(dbus.service.Object):
 
     self.db = sqlite3.connect(SQLITE_DB_FILENAME)
 
-    self.accounts = storage.AccountManager(self.db)
     self.searches = storage.SearchManager(self.db)
     self.streams = storage.StreamManager(self.db)
     self.messages = storage.MessageManager(self.db)
@@ -271,41 +328,23 @@ class Dispatcher(dbus.service.Object):
     self.connection_monitor.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection_monitor.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
-    self.indicator_items = {}
+    self.stream_names = { 'messages': _('Messages'), 'replies': _('Replies'), 'private': _('Private') }
     self.notified_items = []
     self.notified_errors = {}
-    self.messages_indicator = None
-    self.replies_indicator = None
-    self.private_indicator = None
     self.unseen_counts = {}
     for s in "messages", "replies", "private":
       self.unseen_counts[s] = 0
 
-    self.indicate = None 
+    self.mmapp = None
     self.launcher = None
 
     self.job_list = []
 
-    if indicate and util.resources.get_desktop_file():
-      self.indicate = indicate.Server.ref_default ()
-      self.indicate.set_type("message.gwibber")
-      self.indicate.set_desktop_file(util.resources.get_desktop_file())
-      if Dbusmenu and self.indicate:
-        post_menu = Dbusmenu.Menuitem.new ()
-        post_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Update Status"))
-        post_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
-        post_menu.connect("item-activated", self.show_poster)
-
-        menu_server = Dbusmenu.Server.new("/messaging/commands")
-        root = Dbusmenu.Menuitem.new ()
-        root.child_append (post_menu)
-        menu_server.set_root(root)
-        self.indicate.set_menu (menu_server)
-      self.indicate.connect("server-display", self.on_indicator_server_activate)
-      self.indicate.connect("interest-added", self.on_indicator_interest_added)
-      self.indicate.connect("interest-removed", self.on_indicator_interest_removed)
-      self.update_indicators(self.unseen_counts)
-      self.indicate.show()
+    if MessagingMenu:
+      self.mmapp = MessagingMenu.App(desktop_id='gwibber.desktop')
+      self.mmapp.register()
+      self.mmapp.connect('activate-source', self.on_messaging_menu_source_activated)
+      self.update_message_sources(self.unseen_counts)
 
     if Unity and Dbusmenu:
       self.launcher = Unity.LauncherEntry.get_for_desktop_id ("gwibber.desktop")
@@ -321,11 +360,6 @@ class Dispatcher(dbus.service.Object):
       refresh_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
       refresh_menu.connect("item-activated", self.refresh)
       ql.child_append (refresh_menu)
-      accounts_menu = Dbusmenu.Menuitem.new ()
-      accounts_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Accounts"))
-      accounts_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
-      accounts_menu.connect("item-activated", self.show_accounts)
-      ql.child_append (accounts_menu)
       preferences_menu = Dbusmenu.Menuitem.new ()
       preferences_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Preferences"))
       preferences_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
@@ -354,10 +388,13 @@ class Dispatcher(dbus.service.Object):
       # wait a few seconds before alerting the world we are online
       self.refresh_timer_id = GLib.timeout_add_seconds(int(10), self.refresh)
 
-    self.accounts_service = util.getbus("Accounts")
-    self.accounts_service.connect_to_signal("Updated", self.on_account_updated)
-    self.accounts_service.connect_to_signal("Deleted", self.on_account_deleted)
-    self.accounts_service.connect_to_signal("Created", self.on_account_created)
+    self._accounts = []
+    self.account_manager = Accounts.Manager.new_for_service_type("microblogging")
+    self.account_manager.connect("enabled-event", self.on_enabled_event);
+    self.account_manager.connect("account-deleted", self.on_account_deleted)
+    for account in self.account_manager.get_enabled_account_services():
+      self.add_account(Account(account))
+    logger.info ("Found %d accounts", len(self._accounts))
 
     self.streams_service = util.getbus("Streams")
     self.streams_service.connect_to_signal("Created", self.perform_single_op)
@@ -365,6 +402,15 @@ class Dispatcher(dbus.service.Object):
     self.searches_service = util.getbus("Searches")
     self.searches_service.connect_to_signal("Created", self.perform_single_op)
 
+  def get_accounts(self):
+    return [account for account in self._accounts if account.enabled()]
+
+  def get_account(self, id):
+    for account in self._accounts:
+      if account["id"] == id:
+        return account
+    raise KeyError
+
   def show_client(self, stream=None, *args):
     cmd = []
     cmd.append("gwibber")
@@ -377,7 +423,7 @@ class Dispatcher(dbus.service.Object):
     subprocess.Popen("gwibber-preferences", shell=False)
 
   def show_accounts(self, *args):
-    subprocess.Popen("gwibber-accounts", shell=False)
+    subprocess.Popen(["gnome-control-center", "credentials"], shell=False)
 
   def show_poster(self, *args):
     subprocess.Popen("gwibber-poster", shell=False)
@@ -395,7 +441,7 @@ class Dispatcher(dbus.service.Object):
       return False
 
     self.maintRunning = True
-    self.messages.maintenance()
+    self.messages.maintenance(self.account_manager)
     self.maintRunning = False
     self.maintDone = True
     return False
@@ -413,21 +459,32 @@ class Dispatcher(dbus.service.Object):
     if self.refresh_timer_id:
       GLib.source_remove(self.refresh_timer_id)
 
-  def on_account_updated(self, account):
-    self.refresh()
+  def on_enabled_event (self, account_manager, account_id):
+    account = self.account_manager.get_account(account_id)
+    for service in account.list_services():
+      account_service = Accounts.AccountService.new(account, service)
+      if account_service.get_enabled():
+        self.add_account(Account(account_service))
+        self.refresh()
 
-  def on_account_created(self, account):
-    self.refresh()
+  def add_account(self, account):
+    if account in self._accounts:
+      return
+    self._accounts.append(account)
+    logger.info ("Found account %s", account["id"])
 
-  def on_account_deleted(self, account):
+  def on_account_deleted(self, account_manager, account_id):
     # Delete streams associated with the user that was deleted
-    try:
-      acct = json.loads(account)
-      for stream in json.loads(self.streams.List()):
-        if stream["account"] == acct["id"]:
-          self.streams.Delete(stream["id"])
-    except:
-      pass
+      for account in self._accounts:
+        if account.global_id() != account_id: continue
+        logger.debug("Deleting account %s" % account["id"])
+        self._accounts.remove(account)
+        try:
+          for stream in json.loads(self.streams.List()):
+            if stream["account"] == account["id"]:
+              self.streams.Delete(stream["id"])
+        except:
+          pass
 
   def perform_single_op (self, data):
     data = json.loads(data)
@@ -490,7 +547,7 @@ class Dispatcher(dbus.service.Object):
             service = dbus.Interface(obj, "com.Gwibber.Service")
             service.UpdateIndicators("stream")
     """
-    self.handle_indicator_counts(stream)
+    self.handle_message_source_counts(stream)
 
   @dbus.service.method("com.Gwibber.Service", in_signature="s")
   def SendMessage(self, message):
@@ -627,28 +684,6 @@ class Dispatcher(dbus.service.Object):
     logger.info("Gwibber Service is being shutdown")
     self.mainloop.quit()
 
-  @dbus.service.method("com.Gwibber.Service", out_signature="b")
-  def IndicatorInterestCheck(self):
-    """
-    Check for interest from the messaging menu indicator
-    Returns a boolean
-    example:
-            import dbus
-            obj = dbus.SessionBus().get_object("com.Gwibber.Service", "/com/gwibber/Service")
-            service = dbus.Interface(obj, "com.Gwibber.Service")
-            res = service.IndicatorInterestCheck()
-    """
-    if indicate and self.indicate:
-      return self.indicate.check_interest(indicate.INTEREST_SERVER_DISPLAY)
-    else:
-      return False
-
-  @dbus.service.signal("com.Gwibber.Service")
-  def IndicatorInterestAdded(self): pass
-  
-  @dbus.service.signal("com.Gwibber.Service")
-  def IndicatorInterestRemoved(self): pass
-
   @dbus.service.signal("com.Gwibber.Service", signature="s")
   def Error(self, error): pass
 
@@ -728,7 +763,7 @@ class Dispatcher(dbus.service.Object):
             ",".join(self.messages.columns),
             ",".join("?" * len(self.messages.columns))), items)
 
-      GLib.idle_add (self.update_indicators, self.unseen_counts)
+      GLib.idle_add (self.update_message_sources, self.unseen_counts)
       query = """
               SELECT * FROM messages WHERE rowid > {0} AND
                                      ((operation == "receive" AND to_me = 0) OR
@@ -756,44 +791,18 @@ class Dispatcher(dbus.service.Object):
     self.LoadingComplete()
     logger.info("Loading complete: %s - %s", self.refresh_count, output.rowcount if output.rowcount > 0 else 0)
 
-  def update_indicators(self, counts):
+  def update_message_sources(self, counts):
     total_unseen = 0
-    if indicate:
-      if counts.has_key("messages"):
-        if not self.messages_indicator:
-          self.messages_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-          self.messages_indicator.connect("user-display", self.on_indicator_activate)
-          self.messages_indicator.set_property("name", _("Messages"))
-          self.messages_indicator.set_property("stream", "messages")
-          self.messages_indicator.show()
-        self.messages_indicator.set_property("count", str(counts["messages"]))
-        total_unseen += counts["messages"]
-        if self.messages_indicator not in self.indicator_items:
-          self.indicator_items["messages"] = self.messages_indicator
-      if counts.has_key("replies"):
-        if not self.replies_indicator:
-          self.replies_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-          self.replies_indicator.connect("user-display", self.on_indicator_activate)
-          self.replies_indicator.set_property("name", _("Replies"))
-          self.replies_indicator.set_property("stream", "replies")
-          self.replies_indicator.show()
-        self.replies_indicator.set_property("count", str(counts["replies"]))
-        total_unseen += counts["replies"]
-        if self.replies_indicator not in self.indicator_items:
-          self.indicator_items["replies"] = self.replies_indicator
-      if counts.has_key("private"):
-        if not self.private_indicator:
-          self.private_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-          self.private_indicator.connect("user-display", self.on_indicator_activate)
-          self.private_indicator.set_property("name", _("Private"))
-          self.private_indicator.set_property("stream", "private")
-          self.private_indicator.show()
-        self.private_indicator.set_property("count", str(counts["private"]))
-        total_unseen += counts["private"]
-        if counts["private"] > 0:
-          self.private_indicator.set_property_bool("draw-attention", True)
-        if self.private_indicator not in self.indicator_items:
-          self.indicator_items["private"] = self.private_indicator
+    pos = 0
+    if self.mmapp:
+      for source, name in self.stream_names.iteritems():
+        if counts.has_key(source) and counts[source] > 0:
+          if self.mmapp.has_source(source):
+            self.mmapp.set_source_count(source, counts[source])
+          else:
+            self.mmapp.insert_source_with_count(pos, source, None, name, counts[source])
+        total_unseen += counts[source]
+        pos += 1
     if Unity and self.launcher:
       self.launcher.set_property("count", total_unseen)
       if total_unseen < 1:
@@ -802,28 +811,10 @@ class Dispatcher(dbus.service.Object):
         self.launcher.set_property("count_visible", True)
     return False
 
-  def on_indicator_interest_added(self, server, interest):
-    self.IndicatorInterestAdded()
-
-  def on_indicator_interest_removed(self, server, interest):
-    self.IndicatorInterestRemoved()
-
-  def on_indicator_server_activate(self, indicator, timestamp=None):
-    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
-    client_bus = dbus.SessionBus()
-    logger.debug("Raising gwibber client")
-    try:
-      self.handle_indicator_counts()
-    except:
-      pass
-    self.show_client()
-
-  def on_indicator_activate(self, indicator, timestamp=None):
-    if not indicate: return
-    stream = indicator.get_property("stream")
+  def on_messaging_menu_source_activated(self, mmapp, stream):
     logger.debug("Raising gwibber client, focusing %s stream", stream)
     try:
-      self.handle_indicator_counts(stream)
+      self.handle_message_source_counts(stream)
     except:
       pass
     self.show_client(stream=stream)
@@ -834,29 +825,24 @@ class Dispatcher(dbus.service.Object):
   def handle_focus_error(self, *args):
     logger.error("Failed to raise client %s", args)
 
-  def handle_indicator_counts(self, stream=None):
-    if indicate:
+  def handle_message_source_counts(self, stream=None):
+    if self.mmapp:
       if not stream or stream == "home":
-        for s in self.indicator_items.keys():
-          self.indicator_items[s].set_property("count", str(0))
-          self.unseen_counts[s] = 0
-          if s == "private":
-            self.private_indicator.set_property_bool("draw-attention", False)
+        for s in self.stream_names.keys():
+          self.mmapp.remove_source(s)
         if self.launcher:
           self.launcher.set_property("count", 0)
           self.launcher.set_property("count_visible", False)
         return
-      if self.indicator_items.has_key(stream):
-        self.indicator_items[stream].set_property("count", str(0))
-        if stream == "private":
-          self.private_indicator.set_property_bool("draw-attention", False)
+      else:
+        self.mmapp.remove_source(stream)
 
     self.unseen_counts[stream] = 0
     if Unity:
       total_unseen = 0
       for s in self.unseen_counts.keys():
         total_unseen += self.unseen_counts[s]
-      logger.debug ("handle_indicator_counts total_unseen is %d", total_unseen)
+      logger.debug ("handle_message_source_counts total_unseen is %d", total_unseen)
       if self.launcher:
         self.launcher.set_property("count", total_unseen)
         if total_unseen < 1:
@@ -943,7 +929,7 @@ class Dispatcher(dbus.service.Object):
           operations.append(o)
     
       if operations:
-        logger.debug("** Starting Refresh - %s **", mx.DateTime.now())
+        logger.debug("** Starting Refresh - %s **", datetime.now())
         self.LoadingStarted()
         self.perform_async_operation(operations)
 
